/*============================================================================
  File:     CachingExamples.sql

  Summary:  This script shows how to clear cache, see what's in cache and find
		out more about DMVs. Use DMObjects.SQL to create the needed
		functions to access metadata about DMVs.

  Date:     November 2005

  Version:	SQL Server 2005
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

CHECKPOINT				--writes dirty pages to disk
DBCC FREEPROCCACHE		--clears entire plan cache
DBCC DROPCLEANBUFFERS	--clears all clean data cache pages

DECLARE @DBID	smallint
SELECT @DBID = db_id('pubs')
DBCC FLUSHPROCINDB(@DBID)	--clears all clean plan cache for specified database

-- Auto parameterized and cached...
SELECT * 
FROM titles 
WHERE price = $19.99

-- SQL Server 2000/2005
SELECT * FROM master.dbo.syscacheobjects
--    WHERE cacheobjtype = 'Compiled Plan'

-- To use the DMVs, you must be in 90 compatibility mode
-- sp_dbcmptlevel pubs, 90;

-- SQL Server 2005
SELECT cp.*, st.text 
FROM sys.dm_exec_cached_plans AS cp 
	CROSS APPLY sys.dm_exec_sql_text(plan_handle) st 
WHERE st.text like '%titles%'

-- Or even better, you can see number of executions:
SELECT sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
ORDER BY qs.EXECUTION_COUNT DESC

-- Thanks to Bob Beauchemin and his XML query expertise!!!
-- What if we want to look for table scans...
-- We can't just do: where query_plan like '%clustered index scan%'
-- We could:
SELECT sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE CONVERT(NVARCHAR(MAX), query_plan) LIKE '%clustered index scan%'

-- But - even better (thanks to Bob):
SELECT sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE query_plan.exist('//*[@* = "Clustered Index Scan"]') = 1

-- And - EVEN BETTER (thanks again to Bob):
SELECT sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE query_plan.exist('declare default element namespace "http://schemas.microsoft.com/sqlserver/2004/07/showplan";/ShowPlanXML/BatchSequence/Batch/Statements//RelOp[@LogicalOp = "Clustered Index Scan"]') = 1

-- And - to bring it all together as a stored procedure:
CREATE PROCEDURE LookForPhysicalOps (@op VARCHAR(30))
AS
SELECT sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE query_plan.exist('declare default element namespace "http://schemas.microsoft.com/sqlserver/2004/07/showplan";/ShowPlanXML/BatchSequence/Batch/Statements//RelOp/@PhysicalOp[. = sql:variable("@op")]') = 1
GO

EXEC LookForPhysicalOps 'Clustered Index Scan'
EXEC LookForPhysicalOps 'Nested Loops'
EXEC LookForPhysicalOps 'Table Scan'
go

-- So, will subsequent executions use it?
SELECT * 
FROM titles 
WHERE price = $19.99

-- But what if there's a different value?
SELECT * 
FROM titles 
WHERE price = $7.99 -- money

SELECT * 
FROM titles 
WHERE price = 7.99 -- still money?

SELECT * 
FROM titles 
WHERE price = 19.99 -- still the same?

-- Client application
-- Caching by specification, variable is explicitly typed
EXEC sp_executesql
	N'select * from titles where price = @price',
	N'@price money',
	19.99

EXEC sp_executesql
	N'select * from titles where price > @price',
	N'@price money',
	1099.99
